describe('myApp', function() {

    beforeEach(module('myApp'));

    describe('service', function() {
        it('shoud get the sum', inject(function(Calc) {
            expect(Calc.sum(3, 7)).toBe(10);
        }));
    });

    describe('controller', function() {
        var controller,
            Calc;

        beforeEach(inject(function($controller, _Calc_) {
            Calc = _Calc_;
            spyOn(Calc, 'sum').and.returnValue(7);
            controller = $controller('MainController',
                {Calc: Calc});
        }));

        it('should get result', function() {
            controller.doCalculations(3, 4);
            expect(controller.result).toBe(7);
        });

        it('should call Calc service', inject(function() {
            controller.doCalculations(3, 4);
            expect(Calc.sum).toHaveBeenCalled();
        }));

    });

    describe('service with async methods($http)', function() {

        var UserService,
            $httpBackend,
            mockUsers = [{name: 'Bob'}];

        beforeEach(inject(function(_$httpBackend_, _UserService_) {
            UserService = _UserService_;
            $httpBackend = _$httpBackend_;
            $httpBackend.whenGET('/users').respond();
            $httpBackend.whenGET('/users/1').respond(mockUsers[0]);
        }));

        it('should get one user', function(done) {
            UserService.getOne(1).then(function(user) {
                // console.log(111);
                expect(user).toEqual({name: 'Bob'});
                done();
            });

            $httpBackend.flush();
        });

        it('should get list of users', function(done) {
            UserService.getAll(1).then(function(users) {
                expect(users).toEqual(mockUsers);
                done(); // always usable by async requests
            });

            $httpBackend.flush();
        });
    });

});

